//=============================================================================
//  MuseScore
//  Music Composition & Notation
//
//  SqueezeNotes plugin
//
//  Copyright (C)2011 Charles Cave (charlesweb@optusnet.com.au)
//  Copyright (C)2014 Jörn Eichler (joerneichler@gmx.de)
//  Copyright (C)2012-2015 Joachim Schmitz (jojo@schmitz-digital.de)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.2
import MuseScore 1.0

MuseScore {
version:  "1.0"
description: "This plugin change minimal spaces between selected notes (f.e. under slur)"
menuPath: "Plugins.Notes.Min spaces"

property variant gap: 2.5;
    

function getChordsFromSegment(segment){
    var outAr = [];
    // only for 1st and 2nd voices
    for (var voice = 0; voice < 2; voice++){// = cursor.voice;
        var element
        element = segment.elementAt(voice)
        if (element && element.type == Element.CHORD) {
            print("V:"+voice+"|"+element._name() + ": x=" + element.pagePos.x);
            outAr.push(element);  
        }
    }
    return outAr;
}

function shiftHorChord(ch, n){
    // horiz shift all elements of chord (notes, lyrics) 
    ch.pos.x     = -n;
}

function calcNewCoord (a){
    // get array with coordinates
    // calc new coordinates
    // return array of deltas
    var outAr = [];
    var basis = a[0];
    outAr.push(0);
    
    for (var i = 1; i < a.length; i++){
       var delta = 0;
       var xOrig = a[i];
       var xNew = (basis + i*gap);
        outAr.push( xOrig-xNew); 
    }
    
    return outAr;
}
    
// Apply the given function to all chords/rests in selection
// or, if nothing is selected, in the entire score
    function applyToChordsAndRestsInSelection() {
        var cursor = curScore.newCursor()
        cursor.rewind(1)
        var startStaff
        var endStaff
        var endTick
        var fullScore = false
        if (!cursor.segment) { // no selection
            console.log("No selected")
            Qt.quit();
        }
        else {
            startStaff = cursor.staffIdx
            cursor.rewind(2)
            if (cursor.tick == 0) {
                // this happens when the selection includes
                // the last measure of the score.
                // rewind(2) goes behind the last segment (where
                // there's none) and sets tick=0
                endTick = curScore.lastSegment.tick + 1
            }
            else
                endTick = cursor.tick
                endStaff = cursor.staffIdx
        }
        console.log(startStaff + " - " + endStaff + " - " + endTick)
        
        for (var staff = startStaff; staff <= endStaff; staff++) {
            //for (var voice = 0; voice < 4; voice++) {
                var voice = 0;
                cursor.rewind(1) // sets voice to 0
                cursor.voice = voice //voice has to be set after goTo
                cursor.staffIdx = staff

                
                var basisX ;
                var segSelAr = [];// array of arrays [chords] from selected segments
                var segInd = 0;// counter of segments
                while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                    var segment = cursor.segment;
                    if (cursor.element) {
                        if (cursor.element.type == Element.REST){}
                        else if (cursor.element.type == Element.CHORD) {
                            var chord = cursor.element;
                            var notes = chord.notes
                            var note = chord.notes[0];
                            if (note.selected){
                                var segAr = [];//array for chords of segments
                                //print("\tnote selected in S:" + segInd + "|v:"+voice+"|Ch.x = " + chord.pagePos.x);
                                segAr = getChordsFromSegment(segment);
                                segSelAr.push(segAr);
                            }
                            
                        } // end if CHORD
                    } // end if element
                    cursor.next();
                    segInd++;
                } // end while cursor - segments

                if (segSelAr.length == 1){
                    //print("segSelAr.length = "+segSelAr.length);     
                    print("Only one segment selected");
                    Qt.quit();
                }    
                
                
                var xCoordSelChordsAr = [];
                var newXCoordAr = [];
                var deltaXAr = [];
                basisX = segSelAr[0][0].pagePos.x;
                
                print("basis = " + basisX);

                //get x-coord of segments                
                for (var i = 0; i < segSelAr.length; i++) {
                    xCoordSelChordsAr.push(segSelAr[i][0].pagePos.x);
                }
                

                deltaXAr = calcNewCoord(xCoordSelChordsAr);
                
                
                //shift all chords on new coordinates
                for (var i = 1; i < segSelAr.length; i++){
                    print ("Work with "+i+" segment");
                    for (var j = 0; j < segSelAr[i].length; j++){
                        print("\twork with "+j+" Chord");
                        print("\tDelta: = " + deltaXAr[i]);
                        
                        // SHIFT
                        segSelAr[i][j].pos.x =- deltaXAr[i];
                    }
                }
                
            //} // end for loop - voices
        } // end for loop
    }

    onRun: {
        console.log("Start SqueezeNotes.")
        if (typeof curScore !== 'undefined'){
            applyToChordsAndRestsInSelection()
        }            
        Qt.quit()
    }

    
}
